package fishing.particle  
{
	import com.divillysausages.ds.DS;
	import com.divillysausages.ds.particles.IParticle;
	import flash.display.Bitmap;
	import flash.display.BitmapData;
	import flash.geom.Point;
	
	/**
	 * A simple point particle that contains a simple BitmapData object
	 * @author Damian
	 */
	public class ImageParticle extends Bitmap implements IParticle
	{
		
		/*************************************************************************************************************/
		
		private static var M_STAGE_SIZE:Point 	= null; // the width and height of the stage
		private static var M_RENDER_POINT:Point = new Point; // the width and height of the stage
		
		/*************************************************************************************************************/
		
		private var m_visible:Boolean	= false;	// are we currently visible (alive)?
		private var m_force:Point		= new Point; // how much force is acting on us at the minute
		private var m_vel:Point			= new Point; // our vel at this point in time
		private var m_life:Number		= 0.0;		// how long (in seconds) we should live for
		private var m_currLife:Number	= 0.0;		// how long we've currently been alive for
		
		/*************************************************************************************************************/
		
		/**
		 * [read-only] The render point for this ImageParticle
		 */
		public function get renderPoint():Point { return ImageParticle.M_RENDER_POINT; }
		
		/**
		 * Are we currently visible (alive)?
		 */
		override public function get visible():Boolean { return this.m_visible; }
		override public function set visible( b:Boolean ):void 
		{ 
			this.m_visible = b; 
		}
		
		/*************************************************************************************************************/
		
		/**
		 * Creates the ImageParticle
		 */
		public function ImageParticle() 
		{			
			// set the stage size
			if ( ImageParticle.M_STAGE_SIZE == null )
				ImageParticle.M_STAGE_SIZE = new Point( DS.stage.stageWidth, DS.stage.stageHeight );
		}
		
		/**
		 * Inits the IParticle
		 * @param graphics			The list of graphics to use over our life span
		 * @param life				How long should we last for?
		 * @param useGravity		Should we use gravity?
		 */
		public function init( graphics:Vector.<BitmapData>, life:Number = 2.0, useGravity:Boolean = false ):void
		{
			// make sure we have some graphics
			if ( graphics == null || graphics.length == 0 )
			{
				DS.error( this, "There was no graphics passed in for the particle" );
				return;
			}
			this.bitmapData = graphics[0];
			
			// set our life
			this.m_life 	= life;
			this.m_currLife	= 0.0;
		}
		
		/**
		 * Resets the particle
		 */
		public function reset():void
		{
			this.m_currLife = 0.0;
			this.visible	= false;
			this.m_force.x	= this.m_force.y = 0.0;
			this.m_vel.x	= this.m_vel.y = 0.0;
			this.alpha		= 1.0;
		}
		
		/*
		 * Cleans everything up for garbage collection
		 */
		public function destroy():void
		{
			// we should have been removed from the drawing list already, otherwise this will crash the next frame ;)
			this.bitmapData = null; // we didn't create our bitmapdata, so we don't dispose() it
			
			// null everything
			this.m_visible		= false; // will early out in the update (just in case)
			this.m_force		= null;
			this.m_vel			= null;
			
			// the static stuff will still be there, but that's alright, in case we come back to this state
		}
		
		/**
		 * Fires off the particle
		 * @param pos				The starting position of the Particle
		 * @param minRadians		The minimum firing angle, in radians
		 * @param maxRadians		The maximum firing angle, in radians
		 * @param force				How strong should we push it out?
		 * @param emitPointRandom	How much to randomise the emit point by
		 */
		public function fire( pos:Point, minRadians:Number = 0.0, maxRadians:Number = Math.PI * 2.0, force:Number = 100.0, emitPointRandom:Number = 0.0 ):void
		{
			// no graphics, do nothing
			if ( this.bitmapData == null )
				return;
				
			// store the random function for quicker accessing
			var ran:Function = Math.random;
				
			// position us
			if ( emitPointRandom == 0.0 )
			{
				this.x = pos.x;
				this.y = pos.y;
			}
			else
			{
				this.x = pos.x - emitPointRandom + ( emitPointRandom * 2.0 * ran() );
				this.y = pos.y - emitPointRandom + ( emitPointRandom * 2.0 * ran() );
			}
			
			// swap them if they're the wrong way around
			if ( maxRadians < minRadians )
			{
				var r:Number 	= minRadians;
				minRadians		= maxRadians;
				maxRadians		= r;
			}
			
			// get our angle as a random number between the min and the max
			var radians:Number = minRadians + ran() * ( maxRadians - minRadians );
			
			// randomise our force a bit
			force = force * ( 0.8 + ran() * 0.4 );
			
			// set our initial force
			this.m_force.x = Math.cos( radians ) * force;
			this.m_force.y = Math.sin( radians ) * force;
			
			// set our initial vel
			this.m_vel.x = this.m_vel.y = 0.0;
			
			// set our initial life and bitmap data (randomise the life a bit)
			this.m_life		= this.m_life * ( 0.8 + ran() * 0.4 );
			this.m_currLife = 0.0;
			this.alpha		= 1.0;
			
			// we're visible
			this.visible = true;
		}
		
		/**
		 * Updates the particle
		 * @param dt	The time since the last call to update
		 */
		public function update( dt:Number ):void
		{
			if ( !this.m_visible )
				return;
				
			if ( this.bitmapData == null )
				return;
			
			// update our vel
			this.m_vel.x += this.m_force.x * dt;
			this.m_vel.y += this.m_force.y * dt;
			
			// move us along
			this.x += this.m_vel.x;
			this.y += this.m_vel.y;
			
			// reset the force
			this.m_force.x = this.m_force.y = 0.0;
			
			// if we're out of the stage, stop
			if ( this.x < 0.0 || this.y < 0.0 || this.x > ImageParticle.M_STAGE_SIZE.x || this.y > ImageParticle.M_STAGE_SIZE.y )
				this.m_visible 	= false;
				
			// if we're too old, stop
			this.m_currLife += dt;
			if ( this.m_currLife >= this.m_life )
				this.m_visible = false;
				
			// update our alpha
			this.alpha = 1.0 - ( this.m_currLife / this.m_life );
		}
		
	}

}